
# q-expansion of newform 504.2.c.e, downloaded from the LMFDB on 26 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 504
weight = 2
poly_data = [16, 0, -4, 0, -2, 0, -1, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0, 0, 0], 1], [[0, 0, 1, 0, 0, 0, 0, 0], 1], [[0, 0, 0, 1, 0, 0, 0, 0], 1], [[-1, 0, 0, 0, 1, 0, 0, 0], 1], [[0, -4, 0, 0, 0, 1, 0, 1], 4], [[0, 4, 0, 2, 0, 1, 0, -1], 4], [[-6, 0, -2, 0, 1, 0, 1, 0], 2]]

hecke_ring_character_values = [[127, [1, 0, 0, 0, 0, 0, 0, 0]], [253, [-1, 0, 0, 0, 0, 0, 0, 0]], [281, [1, 0, 0, 0, 0, 0, 0, 0]], [73, [1, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 1, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -1, 0, 0], [-1, 0, 0, 0, 0, 0, 0, 0], [0, 2, 0, 0, 0, -1, -1, 0], [0, 0, 2, 0, -1, 0, 0, 1], [0, 2, 0, 2, 0, -1, 0, 0], [0, 0, 0, 0, -1, 0, 0, -1], [0, -2, 0, 2, 0, -1, -2, 0], [0, -2, 0, 0, 0, 2, 1, 0], [2, 0, -2, 0, -1, 0, 0, 1], [0, 0, 0, 0, 1, 0, 0, 1], [0, 2, 0, -2, 0, 1, 2, 0], [0, 0, 2, 0, 0, 0, 0, 2], [0, 0, 0, 0, 0, 0, 0, 0], [0, -2, 0, 0, 0, -2, 1, 0], [0, 4, 0, 0, 0, 0, -2, 0], [0, 0, 2, 0, -1, 0, 0, 1], [0, 0, 2, 0, 0, 0, 0, 2], [0, -2, 0, -2, 0, 1, 0, 0], [-6, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 2, 0, 0], [0, -2, 0, 2, 0, -1, -2, 0], [6, 0, -4, 0, -2, 0, 0, 2], [0, -4, 0, 0, 0, -1, 2, 0], [2, 0, 2, 0, 1, 0, 0, -1], [0, 6, 0, 0, 0, -3, -3, 0], [0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, -4, 0, 2, 2, 0], [4, 0, 4, 0, 2, 0, 0, -2], [0, 0, 0, 0, 0, -2, 0, 0], [0, 4, 0, -4, 0, 2, 4, 0], [0, 0, 4, 0, -4, 0, 0, 0], [0, -6, 0, 0, 0, -2, 3, 0], [-4, 0, -4, 0, -2, 0, 0, 2], [0, 0, -2, 0, 1, 0, 0, -1], [0, 0, -10, 0, 4, 0, 0, -6], [0, -4, 0, -4, 0, 2, 0, 0], [0, 8, 0, 0, 0, 1, -4, 0], [0, 2, 0, 0, 0, 3, -1, 0], [0, 0, 6, 0, -1, 0, 0, 5], [0, -2, 0, -2, 0, 1, 0, 0], [4, 0, 2, 0, 1, 0, 0, -1], [0, 6, 0, 0, 0, -4, -3, 0], [0, 0, 4, 0, 2, 0, 0, -2], [0, 0, -2, 0, 2, 0, 0, 0], [0, 0, -4, 0, -2, 0, 0, 2], [0, 4, 0, 0, 0, -6, -2, 0], [0, 0, -2, 0, 3, 0, 0, 1], [0, 4, 0, 8, 0, -4, -2, 0], [0, 6, 0, -2, 0, 1, 4, 0], [14, 0, -4, 0, -2, 0, 0, 2], [0, -4, 0, 0, 0, -2, 2, 0], [0, -6, 0, -2, 0, 1, -2, 0], [0, 2, 0, 6, 0, -3, -2, 0], [0, 12, 0, 0, 0, -7, -6, 0], [-14, 0, 2, 0, 1, 0, 0, -1], [0, 0, 4, 0, -3, 0, 0, 1], [0, -12, 0, 0, 0, 0, -6, 0], [0, 0, -4, 0, 3, 0, 0, -1], [0, 8, 0, 0, 0, -7, -4, 0], [0, 0, 0, 0, 1, 0, 0, 1], [0, -8, 0, 0, 0, 0, -4, 0], [-14, 0, 0, 0, 0, 0, 0, 0], [0, 2, 0, 0, 0, -2, -1, 0], [0, 0, 10, 0, -4, 0, 0, 6], [14, 0, 4, 0, 2, 0, 0, -2], [0, 2, 0, 0, 0, 7, -1, 0], [0, 0, 6, 0, -5, 0, 0, 1], [0, -2, 0, 2, 0, -1, -2, 0], [0, 2, 0, 2, 0, -1, 0, 0], [-8, 0, -4, 0, -2, 0, 0, 2], [0, 0, -4, 0, 2, 0, 0, -2], [0, 0, -6, 0, 2, 0, 0, -4], [0, 12, 0, -4, 0, 2, 8, 0], [0, 6, 0, 0, 0, 2, -3, 0], [0, 0, -10, 0, 3, 0, 0, -7], [0, 4, 0, -4, 0, 2, 4, 0], [-10, 0, 0, 0, 0, 0, 0, 0], [0, -4, 0, 0, 0, 0, 2, 0], [0, 0, 4, 0, -1, 0, 0, 3], [0, -6, 0, -10, 0, 5, 2, 0], [18, 0, -4, 0, -2, 0, 0, 2], [16, 0, 0, 0, 0, 0, 0, 0], [0, -6, 0, 0, 0, -1, 3, 0], [0, -8, 0, 4, 0, -2, -6, 0], [-22, 0, 0, 0, 0, 0, 0, 0], [0, 12, 0, 0, 0, -5, -6, 0], [-24, 0, 0, 0, 0, 0, 0, 0], [0, -20, 0, 0, 0, 14, 10, 0], [0, 8, 0, -8, 0, 4, 8, 0], [-8, 0, 0, 0, 0, 0, 0, 0], [0, -10, 0, 0, 0, 13, 5, 0], [0, 0, -10, 0, 6, 0, 0, -4], [0, 4, 0, -4, 0, 2, 4, 0], [0, 16, 0, 0, 0, -5, -8, 0], [0, -10, 0, -10, 0, 5, 0, 0], [0, 0, 0, 0, 4, 0, 0, 4], [0, 0, 4, 0, -3, 0, 0, 1], [0, 0, -10, 0, 2, 0, 0, -8], [0, -10, 0, 0, 0, -10, 5, 0], [0, 4, 0, 0, 0, 4, -2, 0], [0, -4, 0, -16, 0, 8, 6, 0], [0, 0, -2, 0, 0, 0, 0, -2], [14, 0, 4, 0, 2, 0, 0, -2], [0, -4, 0, 0, 0, -10, 2, 0], [0, -18, 0, 6, 0, -3, -12, 0], [0, -2, 0, 6, 0, -3, -4, 0], [-10, 0, -8, 0, -4, 0, 0, 4], [-8, 0, -4, 0, -2, 0, 0, 2], [0, 0, 0, 0, 0, 0, 0, 0], [0, 4, 0, 4, 0, -2, 0, 0], [0, 0, -8, 0, 0, 0, 0, -8], [-4, 0, 4, 0, 2, 0, 0, -2], [0, 4, 0, 4, 0, -2, 0, 0], [0, 0, 4, 0, -5, 0, 0, -1], [0, -4, 0, -12, 0, 6, 4, 0], [0, -6, 0, 0, 0, -4, 3, 0], [0, -14, 0, 0, 0, -5, 7, 0], [0, 0, -2, 0, 7, 0, 0, 5], [28, 0, -2, 0, -1, 0, 0, 1], [0, 8, 0, 0, 0, 7, -4, 0], [0, 18, 0, 0, 0, -5, -9, 0], [0, 0, 4, 0, -4, 0, 0, 0], [0, 6, 0, 0, 0, -12, -3, 0], [0, 0, -8, 0, 0, 0, 0, -8], [0, 12, 0, -4, 0, 2, 8, 0], [2, 0, 10, 0, 5, 0, 0, -5], [0, 0, 6, 0, -7, 0, 0, -1], [0, 0, -2, 0, 8, 0, 0, 6], [0, 10, 0, 10, 0, -5, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 4, 0, 0, 4], [0, 2, 0, 2, 0, -1, 0, 0], [2, 0, 4, 0, 2, 0, 0, -2], [0, -12, 0, 0, 0, 11, 6, 0], [0, 0, -4, 0, 4, 0, 0, 0], [0, -16, 0, 0, 0, 5, 8, 0], [0, 0, 0, 4, 0, -2, -2, 0], [0, 0, -8, 0, 4, 0, 0, -4], [0, -10, 0, 0, 0, 2, 5, 0], [-20, 0, 4, 0, 2, 0, 0, -2], [0, 18, 0, 0, 0, 3, -9, 0], [0, 0, -10, 0, 3, 0, 0, -7], [0, 12, 0, 12, 0, -6, 0, 0], [0, 0, -6, 0, -3, 0, 0, -9], [0, 10, 0, 2, 0, -1, 4, 0], [0, 0, 8, 0, -7, 0, 0, 1], [0, 2, 0, -6, 0, 3, 4, 0], [0, 0, -8, 0, 8, 0, 0, 0], [0, 6, 0, -10, 0, 5, 8, 0], [0, 0, -2, 0, 2, 0, 0, 0], [0, -12, 0, -4, 0, 2, -4, 0], [0, 0, -10, 0, 8, 0, 0, -2], [0, 10, 0, -10, 0, 5, 10, 0], [-40, 0, 0, 0, 0, 0, 0, 0], [0, -10, 0, -14, 0, 7, 2, 0], [-26, 0, 8, 0, 4, 0, 0, -4], [0, -20, 0, 0, 0, 7, 10, 0], [0, 14, 0, 0, 0, 9, -7, 0], [0, 8, 0, 0, 0, 0, 4, 0], [28, 0, 4, 0, 2, 0, 0, -2], [0, -16, 0, 0, 0, -8, 8, 0], [0, 20, 0, 0, 0, 0, 10, 0], [0, -8, 0, 8, 0, -4, -8, 0], [36, 0, -4, 0, -2, 0, 0, 2], [0, 0, 6, 0, -1, 0, 0, 5]]
